# A codec for ISO-2022-JP-1 [RFC2237]
# Tamito KAJIYAMA <18 December 2000>

import codecs, japanese.c._japanese_codecs
import re

US_ASCII      = 1
JISX0201_1976 = 2
JISX0208_1978 = 3
JISX0208_1983 = 4
JISX0212_1990 = 5

CHARSETS = {
    "\033(B": US_ASCII,
    "\033(J": JISX0201_1976,
    "\033$@": JISX0208_1978,
    "\033$B": JISX0208_1983,
    "\033$(D": JISX0212_1990,
}

DESIGNATIONS = {}
for k, v in CHARSETS.items():
    DESIGNATIONS[v] = k

re_designations = re.compile("\033(\\([BJ]|\\$[@B]|\\$\\(D)")

class Codec(codecs.Codec):
    encode = japanese.c._japanese_codecs.iso_2022_jp_1_encode
    decode = japanese.c._japanese_codecs.iso_2022_jp_1_decode

class StreamWriter(Codec, codecs.StreamWriter):
    pass

class StreamReader(Codec, codecs.StreamReader):
    def __init__(self, stream, errors='strict'):
        codecs.StreamReader.__init__(self, stream, errors)
        self.data = ''
        self.charset = US_ASCII
    def _read(self, func, size):
        if size == 0:
            return u''
        if size is None or size < 0:
            data = self.data + func()
        else:
            data = self.data + func(max(size, 8) - len(self.data))
        self.data = ''
        if self.charset != US_ASCII:
            data = DESIGNATIONS[self.charset] + data
        pos = data.rfind("\033")
        if pos >= 0 and not re_designations.match(data, pos):
            # data ends on the way of an escape sequence
            data, self.data = data[:pos], data[pos:]
            pos = data.rfind("\033")
        if pos >= 0:
            match = re_designations.match(data, pos)
            if not match:
                raise UnicodeError, "unknown designation"
            self.charset = CHARSETS[match.group()]
            if self.charset in [JISX0208_1978, JISX0208_1983, JISX0212_1990] and \
               (len(data) - match.end()) % 2 == 1:
                data, self.data = data[:-1], data[-1]
            if self.charset != US_ASCII:
                data = data + DESIGNATIONS[US_ASCII]
        return self.decode(data)[0]
    def read(self, size=-1):
        return self._read(self.stream.read, size)
    def readline(self, size=-1):
        return self._read(self.stream.readline, size)
    def readlines(self, size=-1):
        data = self._read(self.stream.read, size)
        buffer = []
        end = 0
        while 1:
            pos = data.find(u'\n', end)
            if pos < 0:
                if end < len(data):
                    buffer.append(data[end:])
                break
            buffer.append(data[end:pos+1])
            end = pos+1
        return buffer
    def reset(self):
        self.data = ''

### encodings module API

def getregentry():
    return (Codec().encode,Codec().decode,StreamReader,StreamWriter)



# Tamito KAJIYAMA <12 June 2001>
# $Id: hgen.py,v 1.3 2002/10/07 16:49:24 kajiyama Exp $

import sys
import os

progname = os.path.basename(sys.argv[0])

__version__ = "1.0"

# N is determined heuristically (must be >= 256)
N = 523

def read(filename, jis_column, ucs_column):
    file = open(filename)
    jis_map = []
    ucs_map = []
    for i in range(N):
        jis_map.append([])
        ucs_map.append([])
    while 1:
        line = file.readline()
        if not line:
            break
        if line[0] == '#':
            continue
        tokens = line.split()
        jis = int(tokens[jis_column], 16) | 0x8080
        ucs = int(tokens[ucs_column], 16)
        jis_map[jis % N].append((jis, ucs))
        ucs_map[ucs % N].append((ucs, jis))
    return jis_map, ucs_map

def dump(prefix, jis_map, ucs_map):
    for n in range(N):
        jis_map[n].sort()
        print "static unsigned char %s_jis_map_%d[] = {" % (prefix, n)
        print "    0x%02x," % len(jis_map[n])
        for jis, ucs in jis_map[n]:
            print "    0x%02x, 0x%02x, 0x%02x," % (jis/N, ucs/256, ucs%256)
        print "};"
    print "static unsigned char *%s_jis_map[] = {" % prefix
    for n in range(N):
        print "    %s_jis_map_%d," % (prefix, n)
    print "};"
    print
    for n in range(N):
        ucs_map[n].sort(lambda x, y: cmp((x[1], x[0]), (y[1], y[0])))
        print "static unsigned char %s_ucs_map_%d[] = {" % (prefix, n)
        print "    0x%02x," % len(ucs_map[n])
        for ucs, jis in ucs_map[n]:
            print "    0x%02x, 0x%02x, 0x%02x," % (ucs/N, jis/256, jis%256)
        print "};"
    print "static unsigned char *%s_ucs_map[] = {" % prefix
    for n in range(N):
        print "    %s_ucs_map_%d," % (prefix, n)
    print "};"

# Stuff not listed in CP932 table, but collected from Win32 API.
ms932_enc_appendix = {
#    ucs: sjis
    0xa1: 0x21,
    0xa6: 0x7c,
    0xa9: 0x63,
    0xaa: 0x61,
    0xab: 0x81e1,
    0xad: 0x2d,
    0xae: 0x52,
    0xaf: 0x8150,
    0xb2: 0x32,
    0xb3: 0x33,
    0xb5: 0x83ca,
    0xb7: 0x8145,
    0xb8: 0x8143,
    0xb9: 0x31,
    0xba: 0x6f,
    0xbb: 0x81e2,
    0xc0: 0x41,
    0xc1: 0x41,
    0xc2: 0x41,
    0xc3: 0x41,
    0xc4: 0x41,
    0xc5: 0x41,
    0xc6: 0x41,
    0xc7: 0x43,
    0xc8: 0x45,
    0xc9: 0x45,
    0xca: 0x45,
    0xcb: 0x45,
    0xcc: 0x49,
    0xcd: 0x49,
    0xce: 0x49,
    0xcf: 0x49,
    0xd0: 0x44,
    0xd1: 0x4e,
    0xd2: 0x4f,
    0xd3: 0x4f,
    0xd4: 0x4f,
    0xd5: 0x4f,
    0xd6: 0x4f,
    0xd8: 0x4f,
    0xd9: 0x55,
    0xda: 0x55,
    0xdb: 0x55,
    0xdc: 0x55,
    0xdd: 0x59,
    0xde: 0x54,
    0xdf: 0x73,
    0xe0: 0x61,
    0xe1: 0x61,
    0xe2: 0x61,
    0xe3: 0x61,
    0xe4: 0x61,
    0xe5: 0x61,
    0xe6: 0x61,
    0xe7: 0x63,
    0xe8: 0x65,
    0xe9: 0x65,
    0xea: 0x65,
    0xeb: 0x65,
    0xec: 0x69,
    0xed: 0x69,
    0xee: 0x69,
    0xef: 0x69,
    0xf0: 0x64,
    0xf1: 0x6e,
    0xf2: 0x6f,
    0xf3: 0x6f,
    0xf4: 0x6f,
    0xf5: 0x6f,
    0xf6: 0x6f,
    0xf8: 0x6f,
    0xf9: 0x75,
    0xfa: 0x75,
    0xfb: 0x75,
    0xfc: 0x75,
    0xfd: 0x79,
    0xfe: 0x74,
    0xff: 0x79,
    0x3094: 0x8394,
    0xf8f0: 0xa0,
    0xf8f1: 0xfd,
    0xf8f2: 0xfe,
    0xf8f3: 0xff,
}

def dump_ms932(sjisfile, j0208file):
    # build ms932 encode/decode map
    ms932 = open(sjisfile).readlines()
    ms932 = [l.split() for l in ms932 if l and l[0] != '#']
    ms932 = [(int(l[0],16), int(l[1], 16)) for l in ms932 if l and l[1][0] != '#']
    ms932_dec = {}
    ms932_enc = ms932_enc_appendix.copy()

    for mbcs, ucs in ms932:
        ms932_dec[mbcs] = ucs
        cur = ms932_enc.get(ucs, 0)
        if cur:
            # Decode to JIS 2-ku and 13-ku rather than IBMNEC/IBM gaiji.
            if cur >= 0x8800:
                ms932_enc[ucs] = mbcs
        else:
            ms932_enc[ucs] = mbcs

    # build JIS0208 encode/decode map
    j0208 = open(j0208file).readlines()
    j0208 = [l.split() for l in j0208 if l and l[0] != '#']
    j0208 = [(int(l[0],16), int(l[2], 16)) for l in j0208 if l and l[2][0] != '#']
    j0208_dec = {}
    j0208_enc = {}
    for mbcs, ucs in j0208:
        j0208_dec[mbcs] = ucs
        j0208_enc[ucs] = mbcs

    jis_map = []
    ucs_map = []

    for i in range(N):
        jis_map.append([])
        ucs_map.append([])

    # build ucs->sjis map
    for ucs, sjis in ms932_enc.iteritems():
        if ucs >= 0x80 and not (0xff61 <= ucs <= 0xff9f):
            if j0208_enc.get(ucs) != sjis:
                ucs_map[ucs % N].append((ucs, sjis))

    # build sjis->ucs map
    for sjis, ucs in ms932_dec.iteritems():
        if sjis >= 0x80 and not (0xa1 <= sjis <= 0xdf):
            if j0208_dec.get(sjis) != ucs:
                jis_map[sjis % N].append((sjis, ucs))

    dump("ms932", jis_map, ucs_map)

def main():
    table1_jis, table1_ucs = read("JIS0208.TXT", 1, 2)
    table2_jis, table2_ucs = read("JIS0212.TXT", 0, 1)
    print "/* This is an auto-generated file (by %s %s) */" % (
        progname, __version__)
    print "/* Do not edit!! */"
    print
    print "#define N", N
    print
    dump("jisx0208", table1_jis, table1_ucs)
    dump("jisx0212", table2_jis, table2_ucs)
    dump_ms932("MS932.TXT", "JIS0208.TXT")
    
if __name__ == "__main__":
    main()



# A codec for ISO-2022-JP-1 [RFC2237]
# Tamito KAJIYAMA <18 December 2000>

import codecs
import re

from japanese.mappings import jis_x_0208, jis_x_0212

US_ASCII      = 1
JISX0201_1976 = 2
JISX0208_1978 = 3
JISX0208_1983 = 4
JISX0212_1990 = 5

CHARSETS = {
    "\033(B": US_ASCII,
    "\033(J": JISX0201_1976,
    "\033$@": JISX0208_1978,
    "\033$B": JISX0208_1983,
    "\033$(D": JISX0212_1990,
}

DESIGNATIONS = {}
for k, v in CHARSETS.items():
    DESIGNATIONS[v] = k

re_designations = re.compile("\033(\\([BJ]|\\$[@B]|\\$\\(D)")

class Codec(codecs.Codec):
    # Unicode to character buffer
    def encode(self, data, errors='strict',
               supported_errors=('strict', 'ignore', 'replace')):
        if errors not in supported_errors:
            raise ValueError, "unknown error handling code: " + str(errors)
        m1 = jis_x_0208.encoding_map
        m2 = jis_x_0212.encoding_map
        buffer = []
        charset = US_ASCII
        for c in u"" + data:
            if c < u'\u0080':
                new_charset = US_ASCII
                s = c.encode("ascii", errors)
            elif c == u'\u00A5': # YEN SIGN
                new_charset = JISX0201_1976
                s = '\\'
            elif c == u'\u203E': # OVERLINE
                new_charset = JISX0201_1976
                s = '~'
            elif m1.has_key(c):
                new_charset = JISX0208_1983
                s = m1[c]
            elif m2.has_key(c):
                new_charset = JISX0212_1990
                s = m2[c]
            elif errors == 'replace':
                new_charset = JISX0208_1983
                s = '\x22\x2e' # U+3013 GETA MARK
            elif errors == 'strict':
                raise UnicodeError, "cannot map \\u%04x to ISO-2022-JP" % ord(c)
            else:
                continue
            if charset != new_charset:
                charset = new_charset
                buffer.append(DESIGNATIONS[charset])
            buffer.append(s)
        if charset != US_ASCII:
            buffer.append(DESIGNATIONS[US_ASCII])
        return (''.join(buffer), len(data))
    # character buffer to Unicode
    def decode(self, data, errors='strict',
               supported_errors=('strict', 'ignore', 'replace')):
        if errors not in supported_errors:
            raise ValueError, "unknown error handling code: " + str(errors)
        m1 = jis_x_0208.decoding_map
        m2 = jis_x_0212.decoding_map
        buffer = []
        data = str(data) # character buffer compatible object
        charset = US_ASCII
        end = 0
        while 1:
            pos = data.find("\033", end)
            if pos < 0:
                if charset != US_ASCII:
                    raise UnicodeError, "malformed input"
                buffer.append(unicode(data[end:], "ascii", errors))
                break
            if charset == US_ASCII:
                buffer.append(unicode(data[end:pos], "ascii", errors))
            elif charset == JISX0201_1976:
                buffer.append(unicode(data[end:pos], "japanese.jis-x-0201-roman", errors))
            else:
                if charset in [JISX0208_1978, JISX0208_1983]:
                    m = m1
                elif charset == JISX0212_1990:
                    m = m2
                for i in range(end, pos, 2):
                    s = data[i:i+2]
                    if m.has_key(s):
                        buffer.append(m[s])
                    elif errors == 'replace':
                        buffer.append(u'\uFFFD') # REPLACEMENT CHARACTER
                    elif errors == 'strict':
                        raise UnicodeError, "unexpected byte 0x%02x found" % ord(data[i])
            match = re_designations.match(data, pos)
            if not match:
                raise UnicodeError, "unknown designation"
            end = match.end()
            charset = CHARSETS[match.group()]
        return (u''.join(buffer), len(data))

class StreamWriter(Codec, codecs.StreamWriter):
    pass

class StreamReader(Codec, codecs.StreamReader):
    def __init__(self, stream, errors='strict'):
        codecs.StreamReader.__init__(self, stream, errors)
        self.data = ''
        self.charset = US_ASCII
    def _read(self, func, size):
        if size == 0:
            return u''
        if size is None or size < 0:
            data = self.data + func()
        else:
            data = self.data + func(max(size, 8) - len(self.data))
        self.data = ''
        if self.charset != US_ASCII:
            data = DESIGNATIONS[self.charset] + data
        pos = data.rfind("\033")
        if pos >= 0 and not re_designations.match(data, pos):
            # data ends on the way of an escape sequence
            data, self.data = data[:pos], data[pos:]
            pos = data.rfind("\033")
        if pos >= 0:
            match = re_designations.match(data, pos)
            if not match:
                raise UnicodeError, "unknown designation"
            self.charset = CHARSETS[match.group()]
            if self.charset in [JISX0208_1978, JISX0208_1983, JISX0212_1990] and \
               (len(data) - match.end()) % 2 == 1:
                data, self.data = data[:-1], data[-1]
            if self.charset != US_ASCII:
                data = data + DESIGNATIONS[US_ASCII]
        return self.decode(data)[0]
    def read(self, size=-1):
        return self._read(self.stream.read, size)
    def readline(self, size=-1):
        return self._read(self.stream.readline, size)
    def readlines(self, size=-1):
        data = self._read(self.stream.read, size)
        buffer = []
        end = 0
        while 1:
            pos = data.find(u'\n', end)
            if pos < 0:
                if end < len(data):
                    buffer.append(data[end:])
                break
            buffer.append(data[end:pos+1])
            end = pos+1
        return buffer
    def reset(self):
        self.data = ''

### encodings module API

def getregentry():
    return (Codec().encode,Codec().decode,StreamReader,StreamWriter)



# Tamito KAJIYAMA <26 September 2001>

import basetests

def test(encoding):
    file = "text.shift_jis"
    basetests.roundrobin_tests(file, encoding)
    basetests.test_backslash_tilde(encoding)
    basetests.test_stream(file, encoding)
    basetests.test_encode(encoding)
    basetests.test_errors(encoding)
    basetests.test_error_handling(encoding, (
        # invalid bytes
        ("abc\x80\x80\x82\xa0", "strict",  None),
        ("abc\x80\x80\x82\xa0", "replace", u"abc\ufffd\u3042"),
        ("abc\x80\x80\x82\xa0", "ignore",  u"abc\u3042"),
        # a character that has no corresponding character in Shift_JIS
        (u"abc\ufffd\u3042", "strict",  None),
        (u"abc\ufffd\u3042", "replace", "abc\x81\xac\x82\xa0"),
        (u"abc\ufffd\u3042", "ignore",  "abc\x82\xa0"),
        ))



# Tamito KAJIYAMA <26 September 2001>

import basetests

def test(encoding):
    file = "text.iso-2022-jp-ext"
    basetests.roundrobin_tests(file, encoding)
    basetests.test_jis_x_0201_roman(encoding)
    basetests.test_stream(file, encoding, 1)
    basetests.test_encode(encoding)
    basetests.test_errors(encoding)
    basetests.test_error_handling(encoding, (
        # invalid bytes in JIS X 0208
        ("abc\033$B\x00\x00\x30\x21\033(B", "strict",  None),
        ("abc\033$B\x00\x00\x30\x21\033(B", "replace", u"abc\ufffd\u4e9c"),
        ("abc\033$B\x00\x00\x30\x21\033(B", "ignore",  u"abc\u4e9c"),
        # invalid byte in JIS X 0201 Roman
        ("abc\033(J\x80xyz\033(B", "strict",  None),
        ("abc\033(J\x80xyz\033(B", "replace", u"abc\ufffdxyz"),
        ("abc\033(J\x80xyz\033(B", "ignore",  u"abcxyz"),
        # invalid byte in JIS X 0201 Katakana
        ("abc\033(I\x80\x31\033(B", "strict",  None),
        ("abc\033(I\x80\x31\033(B", "replace", u"abc\ufffd\uff71"),
        ("abc\033(I\x80\x31\033(B", "ignore",  u"abc\uff71"),
        # invalid bytes in JIS X 0212
        ("abc\033$(D\x00\x00\x30\x21\033(B", "strict",  None),
        ("abc\033$(D\x00\x00\x30\x21\033(B", "replace", u"abc\ufffd\u4e02"),
        ("abc\033$(D\x00\x00\x30\x21\033(B", "ignore",  u"abc\u4e02"),
        # a character that has no corresponding character in ISO-2022-JP-EXT
        (u"abc\ufffd\u4e9c", "strict",  None),
        (u"abc\ufffd\u4e9c", "replace", "abc\033$B\x22\x2e\x30\x21\033(B"),
        (u"abc\ufffd\u4e9c", "ignore",  "abc\033$B\x30\x21\033(B"),
        # unknown designation (GB 2312)
        ("abc\033$A\x30\x21\033(B", "strict",  None),
        ("abc\033$A\x30\x21\033(B", "replace", None),
        ("abc\033$A\x30\x21\033(B", "ignore",  None),
        ))
    file = "supl.iso-2022-jp-ext"
    basetests.roundrobin_tests(file, encoding)



# A codec for JIS X 0201 Katakana
# Tamito KAJIYAMA <4 December 2000>

import codecs
import UserDict

class Codec(codecs.Codec):
    def encode(self,input,errors='strict'):
        return codecs.charmap_encode(input,errors,encoding_map)
    def decode(self,input,errors='strict'):
        return codecs.charmap_decode(input,errors,decoding_map)

class StreamWriter(Codec,codecs.StreamWriter):
    pass
        
class StreamReader(Codec,codecs.StreamReader):
    pass

### encodings module API

def getregentry():
    return (Codec().encode,Codec().decode,StreamReader,StreamWriter)

class Mapping(UserDict.UserDict):
    def __getitem__(self, key):
        try:
            return UserDict.UserDict.__getitem__(self, key)
        except KeyError:
            return None

decoding_map = Mapping()
for c in range(0x21):
    decoding_map[c] = c
for c in range(0x21, 0x60):
    decoding_map[c] = 0xff40 + c
decoding_map[0x7f] = 0x7f

encoding_map = Mapping()
for k, v in decoding_map.items():
    encoding_map[v] = k



# Tamito KAJIYAMA <12 December 2000>

import os
import sys

def remove_module(dir, name):
    for ext in [".py", ".pyc", ".pyo"]:
        path = os.path.join(dir, name + ext)
        if os.path.exists(path):
            os.remove(path)

def main():
    libdir = os.path.join(sys.prefix, 'lib', 'python'+sys.version[:3],
                          'encodings')
    for module in ["euc_jp", "shift_jis", "jis_7",
                   os.path.join("japanese", "__init__"),
                   os.path.join("japanese", "euc2utf"),
                   os.path.join("japanese", "utf2euc"),
                   os.path.join("japanese", "sjis2utf"),
                   os.path.join("japanese", "utf2sjis")]:
        remove_module(libdir, module)
    os.rmdir(os.path.join(libdir, "japanese"))

try:
    main()
except OSError, e:
    sys.stderr.write(str(e) + "\n")



# Tamito KAJIYAMA <26 September 2001>

assert(u"test" == unicode("test", "japanese.euc-jp"))
assert(u"test" == unicode("test", "japanese.shift_jis"))
assert(u"test" == unicode("test", "japanese.iso-2022-jp"))
assert(u"test" == unicode("test", "japanese.iso-2022-jp-1"))
assert(u"test" == unicode("test", "japanese.iso-2022-jp-ext"))
assert(u"test" == unicode("test", "japanese.jis-x-0201-roman"))
assert(u"\uFF83\uFF7D\uFF84" == unicode("C=D", "japanese.jis-x-0201-katakana"))



# Tamito KAJIYAMA <12 March 2000>

import codecs

from japanese.mappings import shift_jis

class Codec(codecs.Codec):
    # Unicode to character buffer
    def encode(self, data, errors='strict',
               supported_errors=('strict', 'ignore', 'replace')):
        if errors not in supported_errors:
            raise ValueError, "unknown error handling code: " + str(errors)
        m = shift_jis.encoding_map
        buffer = []
        for c in u"" + data:
            if c < u'\u0080':
                buffer.append(c.encode("ascii", errors))
            elif c == u'\u00a5': # YEN SIGN
                buffer.append('\\')
            elif c == u'\u203e': # OVERLINE
                buffer.append('~')
            elif m.has_key(c):
                buffer.append(m[c])
            elif c >= u'\uff61' and c <= u'\uff9f':
                buffer.append(self.to_GR(c.encode("japanese.jis-x-0201-katakana", errors)))
            elif errors == 'replace':
                buffer.append('\x81\xac') # U+3013 GETA MARK
            elif errors == 'strict':
                raise UnicodeError, "cannot map \\u%04x to Shift_JIS" % ord(c)
        return (''.join(buffer), len(data))
    # character buffer to Unicode
    def decode(self, data, errors='strict',
               supported_errors=('strict', 'ignore', 'replace')):
        if errors not in supported_errors:
            raise ValueError, "unknown error handling code: " + str(errors)
        m = shift_jis.decoding_map
        buffer = []
        data = str(data) # character buffer compatible object
        size = len(data)
        p = 0
        while p < size:
            if data[p] < "\x80":
                buffer.append(unicode(data[p], "ascii", errors))
                p = p + 1
            elif data[p] >= "\xa1" and data[p] <= "\xdf":
                buffer.append(unicode(self.to_GL(data[p]), "japanese.jis-x-0201-katakana", errors))
                p = p + 1
            else:
                c = data[p:p+2]
                if m.has_key(c):
                    buffer.append(m[c])
                elif errors == 'replace':
                    buffer.append(u'\uFFFD') # REPLACEMENT CHARACTER
                elif errors == 'strict':
                    raise UnicodeError, "unexpected byte 0x%02x found" % ord(data[p])
                p = p + 2
        return (u''.join(buffer), size)
    def to_GL(self, s, func=lambda x, c=chr, o=ord: c(o(x) & 0x7f)):
        return ''.join(map(func, s))
    def to_GR(self, s, func=lambda x, c=chr, o=ord: c(o(x) | 0x80)):
        return ''.join(map(func, s))

class StreamWriter(Codec, codecs.StreamWriter):
    pass

class StreamReader(Codec, codecs.StreamReader):
    def __init__(self, stream, errors='strict'):
        codecs.StreamReader.__init__(self, stream, errors)
        self.data = ''
    def _read(self, func, size):
        if size == 0:
            return u''
        if size is None or size < 0:
            data = self.data + func()
            self.data = ''
        else:
            data = self.data + func(max(size, 2) - len(self.data))
            size = len(data)
            p = 0
            while p < size:
                if data[p] < "\x80" or data[p] >= "\xa1" and data[p] <= "\xdf":
                    p = p + 1
                elif p + 2 <= size:
                    p = p + 2
                else:
                    break
            data, self.data = data[:p], data[p:]
        return self.decode(data)[0]
    def read(self, size=-1):
        return self._read(self.stream.read, size)
    def readline(self, size=-1):
        return self._read(self.stream.readline, size)
    def readlines(self, size=-1):
        data = self._read(self.stream.read, size)
        buffer = []
        end = 0
        while 1:
            pos = data.find(u'\n', end)
            if pos < 0:
                if end < len(data):
                    buffer.append(data[end:])
                break
            buffer.append(data[end:pos+1])
            end = pos+1
        return buffer
    def reset(self):
        self.data = ''

### encodings module API

def getregentry():
    return (Codec().encode,Codec().decode,StreamReader,StreamWriter)



# Tamito KAJIYAMA <2 April 2002>
# $Id: __init__.py,v 1.3 2003/11/29 23:31:23 kajiyama Exp $

import encodings.aliases

encodings.aliases.aliases.update({
    'euc_jp':                 'japanese.euc_jp',
    'eucjp':                  'japanese.euc_jp',
    'ujis':                   'japanese.euc_jp',
    'shift_jis':              'japanese.shift_jis',
    'sjis':                   'japanese.shift_jis',
    'ms932':                  'japanese.ms932',
    'cp932':                  'japanese.ms932',
    'windows_932':            'japanese.ms932',
    'windows_31j':            'japanese.ms932',
    'iso_2022_jp':            'japanese.iso_2022_jp',
    'jis_7':                  'japanese.iso_2022_jp',
    'iso_2022_jp_1':          'japanese.iso_2022_jp_1',
    'iso_2022_jp_ext':        'japanese.iso_2022_jp_ext',
    'jis_x_0201_katakana':    'japanese.jis_x_0201_katakana',
    'jis_x_0201_roman':       'japanese.jis_x_0201_roman',
    })



#!/usr/bin/env python
# Tamito KAJIYAMA <30 November 2000>
# $Id: setup.py,v 1.13 2003/11/29 23:33:07 kajiyama Exp $

from distutils.core import setup, Extension
from distutils.command.build_py import build_py

class Build_py(build_py):
    user_options = build_py.user_options + [
        ('with-aliases', None, "register aliases [default]"),
        ("without-aliases", None, "don't register aliases"),
        ]
    boolean_options = build_py.boolean_options + [
        "with-aliases", "without-aliases",
        ]
    negative_opt = {"without-aliases": "with-aliases"}
    def initialize_options (self):
        build_py.initialize_options(self)
        self.with_aliases = 1
    def finalize_options (self):
        build_py.finalize_options(self)
        if self.with_aliases:
            self.packages.append('japanese.aliases')

import os, sys

if os.sep == '/':
    sitedir = os.path.join("lib", "python" + sys.version[:3], "site-packages")
elif os.sep == ':':
    sitedir = os.path.join("lib", "site-packages")
else:
    if sys.version_info[0:3] >= (2, 2, 0):
        sitedir = os.path.join("lib", "site-packages")
    else:
        sitedir = "."

DESCRIPTION = """\
This package provides Unicode codecs that make Python aware
of Japanese character encodings such as EUC-JP, Shift_JIS and
ISO-2022-JP.  By using this package, Japanese characters can be
treated as a character string instead of a byte sequence."""

LICENSE = "a variant of the BSD license"

setup (name = "JapaneseCodecs",
       version = "1.4.10",
       description = "Japanese Codecs for Python Unicode Support",
       long_description = DESCRIPTION,
       author = "Tamito KAJIYAMA",
       author_email = "RD6T-KJYM@asahi-net.or.jp",
       url = "http://www.asahi-net.or.jp/~rd6t-kjym/python/",
       license = LICENSE, licence = LICENSE,
       platforms = ["anywhere Python runs"],
       cmdclass = {'build_py': Build_py},
       packages = ['japanese',
                   'japanese.python',
                   'japanese.c',
                   'japanese.mappings'],
       data_files = [(sitedir, ['japanese.pth'])],
       ext_modules = [
           Extension("japanese.c._japanese_codecs",
                     ["src/_japanese_codecs.c"])])



# Tamito KAJIYAMA <26 September 2001>

from common_euc_jp import test

test("japanese.c.euc-jp")

# a test of the fix of a buffer overflow bug in japanese.c.euc-jp
for i in range(1000):
    u = unicode("\x8f\xec\xbf\x8f\xe2\xc7" * i, "japanese.c.euc-jp")
    u.encode("japanese.c.euc-jp")



# Tamito KAJIYAMA <26 September 2001>

import basetests

def test(encoding):
    file = "text.euc-jp"
    basetests.roundrobin_tests(file, encoding)
    basetests.test_backslash_tilde(encoding)
    basetests.test_stream(file, encoding)
    basetests.test_encode(encoding)
    basetests.test_errors(encoding)
    basetests.test_error_handling(encoding, (
        # invalid bytes in JIS X 0208
        ("abc\x80\x80\xa4\xa2", "strict",  None),
        ("abc\x80\x80\xa4\xa2", "replace", u"abc\ufffd\u3042"),
        ("abc\x80\x80\xa4\xa2", "ignore",  u"abc\u3042"),
        # invalid byte in JIS X 0201 Katakana
        ("abc\x8e\x00\x8e\xb1", "strict",  None),
        ("abc\x8e\x00\x8e\xb1", "replace", u"abc\ufffd\uff71"),
        ("abc\x8e\x00\x8e\xb1", "ignore",  u"abc\uff71"),
        # invalid bytes in JIS X 0212
        ("abc\x8f\x80\x80\x8f\xb0\xa1", "strict",  None),
        ("abc\x8f\x80\x80\x8f\xb0\xa1", "replace", u"abc\ufffd\u4e02"),
        ("abc\x8f\x80\x80\x8f\xb0\xa1", "ignore",  u"abc\u4e02"),
        # a character that has no corresponding character in EUC-JP
        (u"abc\ufffd\u3042", "strict",  None),
        (u"abc\ufffd\u3042", "replace", "abc\xa2\xae\xa4\xa2"),
        (u"abc\ufffd\u3042", "ignore",  "abc\xa4\xa2"),
        ))
    file = "supl.euc-jp"
    basetests.roundrobin_tests(file, encoding)



# Tamito KAJIYAMA <19 December 2000>

import StringIO
import string
import codecs
import sys

def roundrobin_tests(file, encoding):
    assert unicode(u"\u005C".encode(encoding), encoding) == u"\u005C"
    assert unicode(u"\uFF3C".encode(encoding), encoding) == u"\uFF3C"
    for line in open(file).readlines():
        u = unicode(line, encoding)
        sys.stdout.write(u.encode("utf-8"))
        assert line == u.encode(encoding)

def test_stream(file, encoding, compare_unicode=0):
    encode, decode, Reader, Writer = codecs.lookup(encoding)
    text = open(file).read()
    if compare_unicode:
        test = unicode(text, encoding)
    else:
        test = text
    for name in ["read", "readline", "readlines"]:
        for sizehint in [None, -1] + range(1, 33) + [64, 128, 256, 512, 1024]:
            ###print name, sizehint
            istream = Reader(StringIO.StringIO(text))
            ostream = Writer(StringIO.StringIO())
            func = getattr(istream, name)
            while 1:
                data = func(sizehint)
                if not data:
                    break
                if name == "readlines":
                    ostream.writelines(data)
                else:
                    ostream.write(data)
            if compare_unicode:
                assert test == unicode(ostream.getvalue(), encoding)
            else:
                assert test == ostream.getvalue()

def test_backslash_tilde(encoding):
    assert u"\u00a5\u203e\\~".encode(encoding) == "\\~\\~"

def test_jis_x_0201_roman(encoding):
    # backslash and tilde
    jis = "\033(J\\~\033(B\\~"
    ucs = u"\u00a5\u203e\\~"
    assert unicode(jis, encoding) == ucs
    assert jis == ucs.encode(encoding)
    # control characters
    jis = "\033(J\r\n\033(B\r\n"
    ucs = u"\r\n\r\n"
    assert unicode(jis, encoding) == ucs

def test_encode(encoding):
    encode, decode, Reader, Writer = codecs.lookup(encoding)
    assert encode("abc") == ("abc", 3)
    for obj in [None, 0, [], (), {}]:
        try:
            encode(obj)
        except TypeError:
            pass
        else:
            raise AssertionError, "a TypeError expected"

def test_errors(encoding):
    encode, decode, Reader, Writer = codecs.lookup(encoding)
    for scheme in ["strict", "ignore", "replace", "unknown"]:
        for func, value in [(encode, u""), (decode, "")]:
            try:
                func(value, scheme)
            except ValueError:
                if scheme != "unknown":
                    raise AssertionError, "no error expected"
            else:
                if scheme == "unknown":
                    raise AssertionError, "a ValueError expected"

def test_error_handling(encoding, tests):
    encode, decode, Reader, Writer = codecs.lookup(encoding)
    for source, scheme, expected in tests:
        if type(source) == type(''):
            func = decode
        else:
            func = encode
        if expected:
            result = func(source, scheme)[0]
            assert result == expected
        else:
            try:
                result = func(source, scheme)[0]
            except UnicodeError:
                continue
            raise AssertionError, "a UnicodeError expected"
        if type(result) == type(''):
            sys.stdout.write(escape(result))
        else:
            sys.stdout.write(result.encode("utf-8"))
        sys.stdout.write('\n')

def escape(s):
    buffer = []
    for c in map(ord, s):
        if c < 0x20 or c > 0x7f:
            buffer.append("\\%03o" % c)
        else:
            buffer.append(chr(c))
    return "'" + string.join(buffer, '') + "'"
